;
; analyzes the time traces of the molecules
; that created the peaks that were identified
; with findpeak.
;
; hazen 1/99
;
; modified to give background subtracted value
; based on estimated background from findpeak2.
;
; hazen 3/99
;
; modified to calculate background directly using
; median of surrounding values.
;
; hazen 7/99
;
; above modification removed.
;
; hazen 7/99
;
; modified to use a gaussian weighting when calculating
; the peak intensity in hopes of an improvement in SNR.
;
; hazen 11/99
;
; modified to correct an error in how the background was subtracted
;
; hazen 2/00
;
; made into a procedure to work with batch analysis
;
; Hazen 3/00
;
; This file takes
; input files: image_file
;			image.pks	Peak list for image_file
; output files: image.traces	intensity traces for eack peak
;
; Updated for tiff files
; 
; Charlotte 08/15
;
; Modified by James G 12/15

pro p_nxgn1_ap_CD, run

;print, "you're in greg's world now"
loadct, 5

; generate gaussian peaks

g_peaks = fltarr(2,2,9,9)

for k = 0, 1 do begin
    for l = 0, 1 do begin
       offx = -0.5*float(k)
       offy = -0.5*float(l)
       for i = 0, 8 do begin
         for j = 0, 8 do begin
          dist = 0.4 * ((float(i)-4.0+offx)^2 + (float(j)-4.0+offy)^2)
          g_peaks(k,l,i,j) = 2.0*exp(-dist)
         endfor
       endfor
    endfor
endfor

apeak  = fltarr(9,9)    ; temp storage for analysis

; initialize variables

film_x = fix(1)
film_y = fix(1)
fr_no  = fix(1)

close, 1          ; make sure unit 1 is closed
close, 2
; added file location as run so script can be run independently of others - JG
run = "C:\Users\jgilburt\Documents\SM Aurora A raw files\06 03 17 KC SC dephos CD532\KC SC dephos CD532 2 1 mW 40"
;openr, 1, run + ".pma"
fname= run + ".tif"
openr, 2, run + ".pks"

; figure out size + allocate appropriately

ok = query_tiff(fname, result)
film_l = UINT(result.num_images)

image = read_tiff(fname,image_index=15)
fsize = size(image, /dimensions)
film_x = fsize(0)
film_y = fsize(1)

;result = FSTAT(1)
;readu, 1, film_x
;readu, 1, film_y
;film_l = UINT(DOUBLE(result.SIZE-4)/(DOUBLE(film_x)*DOUBLE(film_y)*2))

;film_l = long(7500)

print, "film x,y,l : ", film_x,film_y,film_l

frame = uintarr(film_x,film_y)

; load the locations of the peaks

foo = fix(0)
x = float(0)
y = float(0)
b = float(0)
no_good = float(0)
good = fltarr(2,4000)
signal = fltarr(4000)
back = fltarr(4000)

while EOF(2) ne 1 do begin
    readf, 2, foo, x, y, s, b
    good(0,no_good) = x
    good(1,no_good) = y
    signal(no_good) = s
    back(no_good) = b
    no_good = no_good + 1
endwhile

flgd = intarr(2,4000)
flgd(0,*) = floor(good(0,*))
flgd(1,*) = floor(good(1,*))

print, no_good , " peaks were found in file"

time_tr = intarr(no_good,film_l)
whc_gpk = intarr(no_good,2)

; calculate which peak to use for each time trace based on
; peak position

for i = 0 , no_good -1 do begin
    whc_gpk(i,0) = round(2.0 * (good(0,i) - flgd(0,i)))
    whc_gpk(i,1) = round(2.0 * (good(1,i) - flgd(1,i)))
endfor

; load the average image

ave_frame = read_tiff(run + "_ave.tif")

; now read values at peak locations into time_tr array

for i = 0, film_l - 1 do begin
    if (i mod 100) eq 0 then print, "working on : ", i, " of", film_l
    frame = read_tiff(fname, image_index = i)
    ;time_tr(0,i) = i
    ;readu, 1, fr_no
    ;time_tr(0,i) = fr_no
    ;readu, 1, frame
    for j = 0, no_good -1 do begin
	 ;weight intensity values around peak by a gaussian function
	 ;subtract background, then integrate weighted peak
       apeak = g_peaks(whc_gpk(j,0),whc_gpk(j,1),*,*) * float(frame(flgd(0,j)-4:flgd(0,j)+4,flgd(1,j)-4:flgd(1,j)+4)-back(j))
       time_tr(j,i) = round(total(apeak))
    endfor
endfor

; subtract background

;for j = 1, no_good - 1 do begin
;   time_tr(j,*) = time_tr(j,*) - round(back(j)*total(g_peaks(whc_gpk(j,0),whc_gpk(j,1),*,*)))
;endfor

; for debugging

;for j = 0, no_good - 1 do begin
;   print, good(0,j), good(1,j)
;   print, whc_gpk(j,0), whc_gpk(j,1)
;   wset, 0
;   contour, ave_frame(flgd(0,j)-3:flgd(0,j)+3,flgd(1,j)-3:flgd(1,j)+3)
;   wset, 1
;   contour, g_peaks(whc_gpk(j,0),whc_gpk(j,1),*,*)
;   A = get_kbrd(1)
;endfor
close, 1
close, 2
close, 3


no_good = no_good - 1
;openw, 1, run + ".traces"
;printf, 1, film_l 
;printf, 1, no_good
;printf, 1, time_tr

;decided to have data exported as csv file as other output format wasn't working in Matlab JG 01/16
runname = run + "x.txt"

 WRITE_CSV, runname , time_tr
  
;writeu, 1, film_l
;writeu, 1, no_good
;writeu, 1, time_tr

close, 1

end