;
; semi-automated routine to find all potential peaks
; in the current image. its sort of implicit that the
; image is 512 x 512...
;
; hazen 1/99
;
; modified to look in the "left" channel for peaks, then
; figure out where the peak should be in the "right" channel,
; and then evaluates both spots to see that they are not to
; close to other spots or otherwise ugly
;
; Hazen 1/99
;
; modified to also to the inverse of the previous comment
; i.e. "right" to "left". also, loads mapping coefficients
; so you have to run calc_mapping3 first.
;
; Hazen 2/99
;
; modified to use the same background subtraction routine
; as findpeak2
;
; Hazen 3/99
;
; modified to map the right half of the screen onto the
; left half of the screen to avoid biases in the histograms
; against peaks that have an intermediate FRET value, i.e.
; half of their intensity is in the left channel and half
; is in the right channel. image must be 512x512.
;
; Hazen 11/99
;
; modified to allow for and find half-integer peak centroid positions
;
; Hazen 11/99
;
; made into a procedure to work with batch analysis
;
; Hazen 3/00
;
; This code goes through an image file and does several things: 
; 1) computes the average image (average through z-stack)
; 2) subtracts background (where background is defined as the
; median value of a 16x16 square)
; 3) match joins two halves of a split screen image to form a
; combined image
; 4) finds peaks which are above a threshold value
;
; input: image file
; output: image_ave.tif	Averaged image (average over z-stack)
;		image_corr.tif	Background subtracted image
;		image_com.tif	Combined LR image with all values < threshold zeroed	
;		image_circle.tif	Image with all accepted peaks circled
;		image.pks		Peaklist for image
;
; Updated for tiff file input and no FRET
; 
; Charlotte 08/15
; 
; Modified by James G 12/15

pro p_nxgn1_ffp_CD, run
;print, "you're in greg's world now"
loadct, 5

COMMON colors, R_ORIG, G_ORIG, B_ORIG, R_CURR, G_CURR, B_CURR

;put a circle shape of diameter 9 into a byte array for use later
circle = bytarr(11,11)

circle(*,0) = [ 0,0,0,0,0,0,0,0,0,0,0]
circle(*,1) = [ 0,0,0,0,1,1,1,0,0,0,0]
circle(*,2) = [ 0,0,0,1,0,0,0,1,0,0,0]
circle(*,3) = [ 0,0,1,0,0,0,0,0,1,0,0]
circle(*,4) = [ 0,1,0,0,0,0,0,0,0,1,0]
circle(*,5) = [ 0,1,0,0,0,0,0,0,0,1,0]
circle(*,6) = [ 0,1,0,0,0,0,0,0,0,1,0]
circle(*,7) = [ 0,0,1,0,0,0,0,0,1,0,0]
circle(*,8) = [ 0,0,0,1,0,0,0,1,0,0,0]
circle(*,9) = [ 0,0,0,0,1,1,1,0,0,0,0]
circle(*,10)= [ 0,0,0,0,0,0,0,0,0,0,0]


; generate gaussian peaks
; generates three gaussian peaks in each of x and y dimensions
; going from -4 to +4, with means of -0.5, 0 and +0.5

g_peaks = fltarr(3,3,9,9)

for k = 0, 2 do begin
    for l = 0, 2 do begin
       offx = 0.5*float(k-1)
       offy = 0.5*float(l-1)
       for i = 0, 8 do begin
         for j = 0, 8 do begin
          dist = 0.4 * ((float(i)-4.0+offx)^2 + (float(j)-4.0+offy)^2)
          g_peaks(k,l,i,j) = exp(-dist)
         endfor
       endfor
    endfor
endfor

; initialize variables

film_x = fix(1)
film_y = fix(1)
fr_no  = fix(1)
fsize = uint(2)

;;; input film
; added file location JG
run = "C:\Users\jgilburt\Documents\SM Aurora A raw files\06 03 17 KC SC dephos CD532\KC SC dephos CD532 2 1 mW 40"
;close, 1          ; make sure unit 1 is closed
;openr, 1, run + ".tif"
fname= run + ".tif"
; figure out size + allocate appropriately

ok = query_tiff(fname, result)
; read first two entries in file, which in this format
; are presumably x and y dimensions
;readu, 1, film_x
;readu, 1, film_y
;print, "film x,y,size : ", film_x, film_y, (result.SIZE)
;film_l = long(long(result.SIZE-4)/(long(film_x)*long(film_y)))
;film_l = UINT(DOUBLE(result.SIZE-4)/(DOUBLE(film_x)*DOUBLE(film_y)*2))
film_l = UINT(result.num_images)
;messy - need to make automaticimage = read_tiff(fname)
image = read_tiff(fname)
fsize = size(image, /dimensions)
film_x = fsize(0)
film_y = fsize(1)

;film_x = 250
;film_y = 250

print, "film x,y,l : ", film_x,film_y,film_l

frame   = uintarr(film_x,film_y)
ave_arr = fltarr(film_x,film_y)

print, "Film length is : ", film_l

     ;compute average image and write it out as a tiff for potential later use
;    for j = 0, 9050 do begin
 ;      readu, 1, fr_no
 ; ;     readu, 1, frame
 ;   endfor

    for j = 0, film_l - 1 do begin
       frame = read_tiff(fname, image_index = j)
       ;readu, 1, fr_no
       ;readu, 1, frame
       ave_arr = ave_arr + frame
    endfor

 ;   close, 1
    ave_arr = ave_arr/float(film_l)
    temp1 = 255*(ave_arr-min(ave_arr))/(max(ave_arr)-min(ave_arr))
    WRITE_TIFF, run + "_ave.tif", temp1, 1, RED = R_ORIG, GREEN = G_ORIG, BLUE = B_ORIG
    ;w=window(dimensions=[250,500])
    tv, temp1

; subtracts background

temp1 = ave_arr                            ;look into this!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
temp1 = smooth(temp1,2,/EDGE_TRUNCATE)
;aves = fltarr(film_x/16,film_y/16)
aves = fltarr(film_x/10,film_y/10)

;for i = 8, film_x, 16 do begin
;   for j = 8, film_y, 16 do begin
for i = 5, film_x, 10 do begin
    for j = 5, film_y, 10 do begin
 ;      aves((i-8)/16,(j-8)/16) = median(temp1(i-8:i+7,j-8:j+7))
       aves((i-5)/10,(j-5)/10) = median(temp1(i-5:i+4,j-5:j+4))
    endfor
endfor


aves = rebin(aves,film_x,film_y)
aves = smooth(aves,20,/EDGE_TRUNCATE)

temp1 = ave_arr - aves
temp2 = 255*(temp1-min(temp1))/(max(temp1)-min(temp1))
;temp1 = frame - (fix(aves) - 20)
;window, 11, xsize = 512, ysize = 512
tv, temp2, 1
WRITE_TIFF, run + "_corr.tif", temp2, 1, RED = R_ORIG, GREEN = G_ORIG, BLUE = B_ORIG


;; need to delete this section for TMR
; open file that contains how the channels map onto each other

;P = fltarr(4,4)
;Q = fltarr(4,4)
;foo = float(1)

;print,      "h:\smFRET\data\beads1.map"
;openr, 1,   "h:\smFRET\data\beads1.map"
;readf, 1, P
;readf, 1, Q
;close, 1

; and map the right half of the screen onto the left half of the screen

;left  = temp1(0:255,0:511)    ;temp1 is background subtracted image
;right = temp1(256:511,0:511)

;right = POLY_2D(right, P, Q, 2)
;combined = left/2 + right/2

; thresholds the image for peak finding purposes

;threshold=50;
;threshold at 0.25v

combined = temp1
temp3 = combined
med = float(median(combined))
;med = float(median(temp1))
print, "med =", med

;; what is this value??
;std = 5       ;for bI5 and hairpin
std =3    ;for Andor IXon
;std = 10       ;for Andor IXon new
threshold = med + std
;print, "threshold =", threshold

for i = 0, film_x - 1 do begin
    for j = 0, film_y - 1 do begin
       if temp3(i,j) lt byte(threshold) then temp3(i,j) = 0
    endfor
endfor

window, 1, xsize = 250, ysize = 250
temp3 = 255*(temp3-min(temp3))/(max(temp3)-min(temp3))
tv, temp3
WRITE_TIFF, run + "_com.tif", temp3, 1, RED = R_ORIG, GREEN = G_ORIG, BLUE = B_ORIG

;tvscl,temp1
;window, 2, xsize = 255, ysize = 512
;tvscl, combined

; find the peaks

temp5 = 255*(temp1-min(temp1))/(max(temp1)-min(temp1))
temp4 = temp3			; combined data

good = fltarr(2,4000)
back = fltarr(4000)
foob = bytarr(7,7)
diff = fltarr(3,3)

no_good = 0

;for i = 12, 232 do begin
    ;for j = 64, 491 do begin
;for i = 64, 192 do begin
    ;for j = 32, 481 do begin

for i = 11, film_x - 12 do begin
    for j = 11, film_y - 12 do begin
; this is a simple threshold, can be altered to suit needs of different sample intensities - usually 25 is good though - JG;
       if temp3(i,j) gt 15  then begin              
      ;if temp3(i,j) gt threshold  then begin
;       print, "near a peak:", i, j

         ; find the nearest maxima

         foob = temp3(i-3:i+3,j-3:j+3)
         z = max(foob, foo)
         y = foo / 7 - 3
         x = foo mod 7 - 3

         ; only analyze peaks in current column,
         ; and not near edge of area analyzed

         if x eq 0 then begin
          if y eq 0 then begin
              y = y + j
              x = x + i

              ; check if its a good peak
              ; i.e. surrounding points below 1 stdev

              quality = 1
              for k = -5, 5 do begin
                 for l = -5, 5 do begin
                   if circle(k+5,l+5) gt 0 then begin
                    if temp3(x+k,y+l) gt byte(med + 0.5*float(z)) then quality = 0
                    ;print, "threshold2=", byte(med + 0.5*float(z)), temp3(x+k,y+l), quality
                   endif
                 endfor
              endfor

              if quality eq 1 then begin
              print, "found good peak", x, y
                 ; draw where peak was found on screen

                 for k = -5, 5 do begin
                   for l = -5, 5 do begin
                    if circle(k+5,l+5) gt 0 then begin
                        temp5(x+k,y+l) = 90
                        temp4(x+k,y+l) = 90
                    endif
                   endfor
                 endfor

                 ; compute difference between peak and gaussian peak

                 cur_best = 200000.0
                 best_x = x
                 best_y = y
                 print, x, y
                 for k = 0, 2 do begin
                   for l = 0, 2 do begin
                    diff(k,l) = total(abs((temp1(x,y) - aves(x,y)) * g_peaks(k,l,*,*) - (float(temp1(x-4:x+4,y-4:y+4)) - aves(x,y))))
                    ;print, "diff=", diff(k,l)
                    if diff(k,l) lt cur_best then begin
                        best_x = k
                        best_y = l
                        cur_best = diff(k,l)
                        ;print, "cur_best=", cur_best
                    endif
                   endfor
                 endfor

                 flt_x = float(x) - 0.5*float(best_x-1)
                 flt_y = float(y) - 0.5*float(best_y-1)
                 print, flt_x, flt_y
			
			 ;; need to remove this for TMR code but not no_good update
                 ; calculate and draw location of companion peak

                 ;xf = 256.0
                 ;yf = 0.0
                ; for k = 0, 3 do begin
                 ;  for l = 0, 3 do begin
                  ;  xf = xf + P(k,l) * float(flt_x^l) * float(flt_y^k)
                   ; yf = yf + Q(k,l) * float(flt_x^l) * float(flt_y^k)
                   ;endfor
                 ;endfor

                 ;int_xf = round(xf)
                 ;int_yf = round(yf)

                 ;if int_xf+k le film_x-1 and int_xf-k ge 0 and int_yf+l le film_y-1 and int_yf-l ge 0 then begin

                   ;for k = -5, 5 do begin
                   ;  for l = -5, 5 do begin
                   ;   if circle(k+5,l+5) gt 0 then begin
                   ;       temp5(int_xf+k,int_yf+l) = 90
                   ;   endif
                   ;  endfor
                   ;endfor

                   ;xf = float(round(2.0 * xf)) * 0.5
                   ;yf = float(round(2.0 * yf)) * 0.5

                 wset, 0
                 tv, temp5
                 wset, 1
                 tv, temp4
                 WRITE_TIFF, run + "_circle.tif", temp4, 1, RED = R_ORIG, GREEN = G_ORIG, BLUE = B_ORIG

                 good(0,no_good) = flt_x
                 good(1,no_good) = flt_y
                 back(no_good) = aves(x,y)
                 no_good = no_good + 1
                 ;good(0,no_good) = xf
                 ;good(1,no_good) = yf
                 ;back(no_good) = aves(int_xf,int_yf)
                 ;no_good = no_good + 1
                 ;endif
              endif
          endif
         endif
       endif
    endfor
endfor

print, "there were ", no_good, "good peaks"

; write peaklist to file
close, 1
openw, 1, run + ".pks"
for i = 0, no_good - 1 do begin
    printf, 1, i+1, good(0,i), good(1,i), ave_arr(good(0,i), good(1,i)), back(i)
endfor

close, 1
end